<?php
/*----------------------------------------------------------------------------\
|                    Wordsmith - Weblog & Publishing Tool                     |
|                             Version 1.1 BETA                                |
|-----------------------------------------------------------------------------|
|                         Created by Tomas kohl                               |
|                      (http://tomaskohl.com/wordsmith)                       |
|                    Copyright (c) 2003 - 2004 Tomas Kohl                     |
|-----------------------------------------------------------------------------|
| This software is provided "as is", without warranty of any kind, express or |
| implied, including  but not limited  to the warranties of  merchantability, |
| fitness for a particular purpose and noninfringement. In no event shall the |
| authors or  copyright  holders be  liable for any claim,  damages or  other |
| liability, whether  in an  action of  contract, tort  or otherwise, arising |
| from,  out of  or in  connection with  the software or  the  use  or  other |
| dealings in the software.                                                   |
|                                                                             | 
| This  software is  available under the BSD license                          |
|----------------------------------------------------------------------------*/

$xmlrpc_methods = array();
$xmlrpc_methods['method_not_found']          = 'XMLRPC_method_not_found';
$xmlrpc_methods['blogger.getUsersBlogs']     = 'wsBlogInfo';
$xmlrpc_methods['metaWeblog.getRecentPosts'] = 'wsGetRecentPosts';
$xmlrpc_methods['metaWeblog.getCategories']  = 'wsGetPostTopics';
$xmlrpc_methods['metaWeblog.getPost']        = 'wsGetSinglePost';
$xmlrpc_methods['metaWeblog.newPost']        = 'wsNewPost';
$xmlrpc_methods['metaWeblog.editPost']       = 'wsEditPost';
$xmlrpc_methods['metaWeblog.newMediaObject'] = 'wsUploadObject';

function XMLRPC_method_not_found($methodName)
{
  XMLRPC_error("1", "The method you requested, '".$methodName."', was not found.", WEBLOG_XMLRPC_USERAGENT);
}

function _wsSignOn($params)
{// don't expose this method!
  GLOBAL $Blog;
  $uname = $params[1];
  $upass = $params[2];
  if(($uData = $Blog->login($uname, $upass)) != false)
    return true;
  return false;
}

function wsBlogInfo($params)
{
  GLOBAL $Blog;
  $uname = $params[1];
  $upass = $params[2];
  $uData = $Blog->login($uname, $upass);
  $bIsAdmin = ($uData['is_admin'] == '1' ? 1:0);
  $ret = array(
    0=>array(
    "isAdmin" => $bIsAdmin,
    "url"     => $Blog->settings['site_url'],
    "blogid"  => 1,
    "blogName"=> $Blog->settings['site_name']
    )
  );
  XMLRPC_response(XMLRPC_prepare($ret), WEBLOG_XMLRPC_USERAGENT);
}

function  wsGetPostTopics($params)
{
  GLOBAL $Blog;
  $topics = $Blog->listTopics();
  $output = array();
  if(is_array($topics))
  {
    if(count($topics) > 0)
    {
      while(list($id, $topic) = each($topics))
      {
        $output[] = array(
          'categoryId'=>$id,
          'categoryName'=>$topic,
          'description'=>$topic, //until Wordsmith supports Category description
          'htmlUrl'=>$Blog->settings['site_url'].'/archive/'.tweakFilename($topic).$Blog->settings['archive_post_ext'],
          'rssUrl'=>'' //until Wordsmith supports Category syndication
        );
      }
      XMLRPC_response(XMLRPC_prepare($output), WEBLOG_XMLRPC_USERAGENT);
    }
    else
      XMLRPC_error("1", "metaWeblog.getCategories: There are no categories.", WEBLOG_XMLRPC_USERAGENT);
  }
  else
    XMLRPC_error("1", "metaWeblog.getCategories: Database is down at the moment, or the query didn't parse.", WEBLOG_XMLRPC_USERAGENT);
}

function wsGetRecentPosts($params)
{
  GLOBAL $Blog;
  $filter = array();
  $page = 1;
  $limit = $params[3];
  $output = array();

  $ResultSet = $Blog->search($filter,$page,$limit);
  if(is_object($ResultSet))
  {
    while(!$ResultSet->EOF)
    {
      $mdate = strtotime($ResultSet->fields['created_on']);
      $isoDate = date('Ymd',$mdate).'T'.date('H:i:s',$mdate);

      $output[] = array(
        'postid'=>$ResultSet->fields['id'],
        'author'=>$ResultSet->fields['first_name'].' '.$ResultSet->fields['last_name'],
        'dateCreated'=>$isoDate,
        'title'=>$ResultSet->fields['title'],
        'description'=>$ResultSet->fields['story'],
        'link'=> ($Blog->settings['site_url'].'/archive/'.$ResultSet->fields['id'].$Blog->settings['archive_post_ext']
        )
      );
      $ResultSet->MoveNext();
    }

    XMLRPC_response(XMLRPC_prepare($output), WEBLOG_XMLRPC_USERAGENT);
  }
  else
  {
    XMLRPC_error("1", "metaWeblog.getRecentPosts: Database is down at the moment, or the query didn't parse.", WEBLOG_XMLRPC_USERAGENT);
  }
}

function wsGetSinglePost($params)
{
  GLOBAL $Blog;
  $entry = $Blog->getSinglePost($params[0]);
  if(!empty($entry) && is_array($entry))
  {
    $entryCategories = $Blog->listStoryTopics($params[0]);
    $mdate = strtotime($entry['created_on']);
    $isoDate = date('Ymd',$mdate).'T'.date('H:i:s',$mdate);
    $output = array(
      'postid'=>$entry['id'],
      'link' => ($Blog->settings['site_url'].'/archive/'.$entry['id'].$Blog->settings['archive_post_ext']),
      'title' => $entry['title'],
      'description' => $entry['story'],
      'dateCreated' => $isoDate,
      'userid' => $entry['author_id'],
      'content' => $entry['story'].$entry['story_extended'],
      'permalink' => ($Blog->settings['site_url'].'/archive/'.$entry['id'].$Blog->settings['archive_post_ext']),
      'categories' => array_values($entryCategories),//?
      'mt_excerpt' => $entry['abstract'],
      'mt_allow_comments' => $entry['allow_comments'],
      'mt_allow_pings' => $entry['allow_pings'],
      'mt_text_more' => $entry['story_extended']
    );

    XMLRPC_response(XMLRPC_prepare($output), WEBLOG_XMLRPC_USERAGENT);
  }
  else
    XMLRPC_error("1", "metaWeblog.getPost: Database is down at the moment, or the query didn't parse.", WEBLOG_XMLRPC_USERAGENT);
}

function wsNewPost($params)
{
  //TODO: convert all text fields to UTF-8; enable trackback
  GLOBAL $Blog;
  $uData = $Blog->login($params[1], $params[2]);
  if(empty($uData) || empty($uData['id']))
    XMLRPC_error("1", "metaWeblog.newPost: Could not authenticate user.", WEBLOG_XMLRPC_USERAGENT);
  else
  {
    $npost = $params[3];
    $topicNames = $npost['categories'];
    $topicIDs = array();
    if(is_array($topicNames))
    {//move to a new Blog class method
      foreach($topicNames as $topic)
      {
        $sql = "SELECT id FROM cs_topics WHERE topic='".$topic."'";
        $res = $Blog->db->execute($sql);
        if(is_object($res) && !empty($res->fields['id']))
          $topicIDs[] = $res->fields['id'];
      }
    }
    $entry = array(
        'id' => $npost['id'],
        'author_id'=>$uData['id'],
        'title'=>addslashes(stripslashes($npost['title'])),
        'story'=>addslashes(stripslashes($npost['description'])),
        'story_extended'=>addslashes(stripslashes($npost['mt_text_more'])),
        'status'=>(empty($params[4])?1:2),
        'topic'=>$topicIDs,
        'allow_comments'=>$Blog->settings['allow_comments'],
        'allow_pings'=>$Blog->settings['allow_pings']
    );
    $newEntryId = $Blog->post($entry);
    if($newEntryId !== false)
    {
      // caching
      if($entry['status'] == 2)
      {
        cache_single($Blog,$newEntryId,'./');
        cache_main($Blog,$newEntryId,'./');
        if(!empty($topicIDs) && is_array($topicIDs))
        {
          while(list($topID,)=each($storyTopics))
            cacheByTopic($Blog, $topID, './');
        }
      }
      // response
      $output = (empty($entry['id'])) ? $newEntryId:true; // wsEditPost is supposed to return boolean
      XMLRPC_response(XMLRPC_prepare($output), WEBLOG_XMLRPC_USERAGENT);
    }
    else
    {
      XMLRPC_error("1", "metaWeblog.newPost: Could not save your entry.", WEBLOG_XMLRPC_USERAGENT);
    }
  }
}

function wsEditPost($params)
{
  GLOBAL $Blog;
  $uData = $Blog->login($params[1], $params[2]);
  if(empty($uData) || empty($uData['id']))
    XMLRPC_error("1", "metaWeblog.editPost: Could not authenticate user.", WEBLOG_XMLRPC_USERAGENT);
  else
  {
    $currentEntry = $Blog->getSinglePost($params[0]);
    if($uData['id'] != $currentEntry['author_id'] && $uData['is_admin'] != '1')
      XMLRPC_error("1", "metaWeblog.editPost: Could cannot edit this post since you haven't authored it and you don't possess administrative priviledges.", WEBLOG_XMLRPC_USERAGENT);
    else
    {
      $params[3]['id'] = $params[0];
      wsNewPost($params);
    }
  }
}


function wsUploadObject($params)
{
  //TODO: implement this method as soon as wbloggar supports it
  XMLRPC_error("1", "metaWeblog.wsUploadObject: Not implemented yet.", WEBLOG_XMLRPC_USERAGENT);
}
?>