<?php
/*----------------------------------------------------------------------------\
|                    Wordsmith - Weblog & Publishing Tool                     |
|                             Version 1.1 BETA                                |
|-----------------------------------------------------------------------------|
|                         Created by Tomas kohl                               |
|                      (http://tomaskohl.com/wordsmith)                       |
|                    Copyright (c) 2003 - 2004 Tomas Kohl                     |
|-----------------------------------------------------------------------------|
| This software is provided "as is", without warranty of any kind, express or |
| implied, including  but not limited  to the warranties of  merchantability, |
| fitness for a particular purpose and noninfringement. In no event shall the |
| authors or  copyright  holders be  liable for any claim,  damages or  other |
| liability, whether  in an  action of  contract, tort  or otherwise, arising |
| from,  out of  or in  connection with  the software or  the  use  or  other |
| dealings in the software.                                                   |
|                                                                             |
| This  software is  available under the BSD license                          |
|----------------------------------------------------------------------------*/
$xmlrpc_methods = array();
$xmlrpc_methods['method_not_found'] = 'XMLRPC_method_not_found';
$xmlrpc_methods['wordsmith.signOn'] = 'wsSignOn';
$xmlrpc_methods['wordsmith.searchPosts'] = 'wsSearchPosts';
$xmlrpc_methods['wordsmith.searchFeedback'] = 'wsSearchCommentsTrackbacks';
$xmlrpc_methods['wordsmith.searchTracker'] = 'wsSearchTracker';
$xmlrpc_methods['wordsmith.listBookmarkCategories'] = 'wsListLinkCategories';
$xmlrpc_methods['wordsmith.listBookmarks'] = 'wsListLinks';
$xmlrpc_methods['wordsmith.isBookmarkFresh'] = 'wsIsBookmarkFresh';

function XMLRPC_method_not_found($methodName)
{
  XMLRPC_error("1", "The method you requested, '".$methodName."', was not found.", WEBLOG_XMLRPC_USERAGENT);
}

function wsSignOn($params)
{
  GLOBAL $Blog;
  $uname = $params[0];
  $upass = $params[1];
  if(($uData = $Blog->login($uname, $upass)) != false)
  {
    $newSessId = wsNewSessId();
    if(wsNewSession($Blog->db, $uData['id'], $newSessId))
      return XMLRPC_response(XMLRPC_prepare($newSessId), WEBLOG_XMLRPC_USERAGENT);
    else
      return XMLRPC_error("1", "wordsmith.signOn() error: Could not create a new session.", WEBLOG_XMLRPC_USERAGENT);
  }
  else
    return XMLRPC_error("1", "wordsmith.signOn() error: Could not login using provided credentials.", WEBLOG_XMLRPC_USERAGENT);
}

function wsSearchPosts($params)
{
  GLOBAL $Blog;
  $filter = trimall($params[1]);
  $page = $params[2];
  $limit = $params[3];

  $ResultSet = $Blog->search($filter,$page,$limit);
  if(is_object($ResultSet))
  {
    $posts = $ResultSet->getArray();
    $postsFiltered = array();
    foreach($posts as $row)
    {
      $postsFiltered[] = array(
        'id'          => $row['id'],
        'author_id'   => $row['author_id'],
        'first_name'  => $row['first_name'],
        'last_name'   => $row['last_name'],
        'status'      => $row['status'],
        'status_name' => $row['status_name'],
        'title'       => $row['title'],
        'created_on'  => $row['created_on'],
        'comments'    => $row['comments'],
        'trackbacks'  => $row['trackbacks']
      );
    }
    for($i=0;$i<count($postsFiltered);$i++) 
      $postsFiltered[$i]['views'] = $Blog->getViewsPerPost($postsFiltered[$i]['id'], false);
    $recTotal = $ResultSet->MaxRecordCount();
    $output = array(0=>$recTotal,1=>$postsFiltered);
    XMLRPC_response(XMLRPC_prepare($output), WEBLOG_XMLRPC_USERAGENT);
  }
  else
  {
    XMLRPC_error("1", "wordsmith.searchPosts() error: Database is down at the moment, or the query didn't parse.", WEBLOG_XMLRPC_USERAGENT);
  }
}

function wsSearchCommentsTrackbacks($params)
{
  GLOBAL $Blog;
  $type = $params[1];
  if(!in_array($type, array('comments', 'trackbacks')))
    $type = 'comments';
  $criteria = $params[2];
  if(!is_array($criteria))
    XMLRPC_error("1", "wordsmith.searchFeedback() error: incorrect parameter #2. Expecting array, got ".gettype($criteria).".", WEBLOG_XMLRPC_USERAGENT);
  $page  = (int) $params[3];
  $limit = (int) $params[4];
  $htmlComment = $params[5];
  $ResultSet = $Blog->searchFeedback($type,$criteria,$page,$limit);
  if(is_object($ResultSet))
  {
    $data = $ResultSet->getArray();
    if($htmlComment == false)
    {
      for($i=0;$i<count($data);$i++)
        $data[$i]['comment'] = strip_tags($data[$i]['comment']);
    }
    $recTotal = $ResultSet->MaxRecordCount();
    $output = array(0=>$recTotal,1=>$data);
    XMLRPC_response(XMLRPC_prepare($output), WEBLOG_XMLRPC_USERAGENT);
  }
  else
  {
    XMLRPC_error("1", "wordsmith.searchFeedback() error: Database is down at the moment, or the query didn't parse.", WEBLOG_XMLRPC_USERAGENT);
  }
}   


function wsSearchTracker($params)
{
  GLOBAL $Blog;
  $fs = trimall($params[1]);
  $ft = trimall($params[2]);
  $limit = $params[3];
  $page = $params[4];
  $ResultSet = $Blog->searchTracker($fs,$ft,$page,$limit);
  if(is_object($ResultSet))
  {
    $data = $ResultSet->getArray();
    $recTotal = $ResultSet->MaxRecordCount();
    $output = array(0=>$recTotal,1=>$data);
    XMLRPC_response(XMLRPC_prepare($output), WEBLOG_XMLRPC_USERAGENT);
  }
  else
  {
    XMLRPC_error("1", "wordsmith.searchTracker() error: Database is down at the moment, or the query didn't parse.", WEBLOG_XMLRPC_USERAGENT);
  }
}

function wsListLinkCategories($params)
{
  GLOBAL $Blog;
  $activeCatsOnly = $params[0];
  $cats = $Blog->listBookmarkCats();
  if(is_array($cats))
  {
    if($activeCatsOnly == false)
      XMLRPC_response(XMLRPC_prepare($cats), WEBLOG_XMLRPC_USERAGENT);
    else
    {
      $list = array();
      foreach($cats as $row)
      {
        if($row['bookmarks'] > 0)
          $list[] = $row;
      }
      XMLRPC_response(XMLRPC_prepare($list), WEBLOG_XMLRPC_USERAGENT);
    }
  }
  else
    XMLRPC_error("1", "wordsmith.listBookmarkCategories() error: Database is down at the moment, or the query didn't parse.", WEBLOG_XMLRPC_USERAGENT);
}

function wsListLinks($params)
{
  GLOBAL $Blog;
  $categoryId = (int) $params[0];
  $list = $Blog->listBookmarksForCategory(1, $categoryId);
  if(is_array($list))
    XMLRPC_response(XMLRPC_prepare($list), WEBLOG_XMLRPC_USERAGENT);
  else
    XMLRPC_error("1", "wordsmith.listBookmarks() error: Database is down at the moment, or the query didn't parse.", WEBLOG_XMLRPC_USERAGENT);
}

function wsIsBookmarkFresh($params)
{
  GLOBAL $db;
  $sql = "SELECT rss_file FROM cs_neighborhood WHERE uri='".$params[0]."'";
  $res = $db->execute($sql);
  if(!empty($res->fields['rss_file']))
  {
    $today = getdate();
    $date = mktime(0,0,0,$today['mon'], $today['mday'], $today['year']);
    $rss = fetch_rss($res->fields['rss_file']);
    $published = parse_w3cdtf($rss->items[0]['dc']['date']);
    $isCurrent = ($published >= $date) ? "Y":"N";
    XMLRPC_response(XMLRPC_prepare($isCurrent), WEBLOG_XMLRPC_USERAGENT);
  }
  else
    XMLRPC_error("1", "wordsmith.isBookmarkFresh() error: Database is down at the moment, or the query didn't parse.", WEBLOG_XMLRPC_USERAGENT);
}
?>