# WORDSMITH 1.1 BETA database schema
# --------------------------------------------------------

#
# Table structure for table `cs_author`
#

CREATE TABLE cs_author (
  id smallint unsigned NOT NULL auto_increment,
  first_name varchar(128) NOT NULL,
  last_name varchar(128) NOT NULL,
  city varchar(255),
  country varchar(255),
  timezone varchar(255),
  email varchar(255),
  login varchar(128),
  passwd char(32),
  is_admin char(1) NOT NULL default '0',
  created_on datetime,
  PRIMARY KEY  (id),
  UNIQUE KEY login (login)
) TYPE=MyISAM;

# default author/login
INSERT INTO cs_author(id, first_name, last_name, login, passwd, is_admin, created_on) VALUES (1,'John','Doe','admin','5f4dcc3b5aa765d61d8327deb882cf99',1,now());

# --------------------------------------------------------

#
# Table structure for table `cs_settings`
#

CREATE TABLE cs_settings (
  param_name varchar(255) NOT NULL,
  param_value varchar(255) NOT NULL,
  PRIMARY KEY  (param_name)
) COMMENT='settings for weblog';

# default settings
INSERT INTO cs_settings (param_name, param_value) VALUES ('site_name', 'Your weblog') ;
INSERT INTO cs_settings (param_name, param_value) VALUES ('date_format', 'Y-m-d') ;
INSERT INTO cs_settings (param_name, param_value) VALUES ('archive_post_ext', '.html') ;
INSERT INTO cs_settings (param_name, param_value) VALUES ('auto_archive_after', '3') ;
INSERT INTO cs_settings (param_name, param_value) VALUES ('auto_archive_unit', 'MONTH') ;
INSERT INTO cs_settings (param_name, param_value) VALUES ('headlines_no', '10') ;
INSERT INTO cs_settings (param_name, param_value) VALUES ('days_per_page', '14') ;
INSERT INTO cs_settings (param_name, param_value) VALUES ('days_per_syndication', '10') ;
INSERT INTO cs_settings (param_name, param_value) VALUES ('use_topics', '1') ;
INSERT INTO cs_settings (param_name, param_value) VALUES ('show_tips', '1') ;
INSERT INTO cs_settings (param_name, param_value) VALUES ('use_syndication', '1') ;
INSERT INTO cs_settings (param_name, param_value) VALUES ('allow_comments', '1') ;
INSERT INTO cs_settings (param_name, param_value) VALUES ('allow_pings', '1') ;
INSERT INTO cs_settings (param_name, param_value) VALUES ('ping_sites', 'Weblogs.com;Wordsmith') ;
INSERT INTO cs_settings (param_name, param_value) VALUES ('handle_banned', 'BAN_INTERACT') ;

# --------------------------------------------------------

#
# Table structure for table `cs_story_status`
#

CREATE TABLE cs_story_status (
  id tinyint unsigned NOT NULL auto_increment,
  status varchar(127) NOT NULL,
  description varchar(255),
  PRIMARY KEY  (id),
  UNIQUE KEY status (status)
) TYPE=MyISAM;

# 5 implemented statuses - DO NOT CHANGE!
INSERT INTO cs_story_status (id, status, description) VALUES (1, 'Draft', 'Story draft') ;
INSERT INTO cs_story_status (id, status, description) VALUES (2, 'Published', 'Published story') ;
INSERT INTO cs_story_status (id, status, description) VALUES (3, 'Scheduled', 'Story scheduled for publishing') ;
INSERT INTO cs_story_status (id, status, description) VALUES (4, 'Archived', 'Archived story - no modifications or comments possible') ;
INSERT INTO cs_story_status (id, status, description) VALUES (5, 'Deleted', 'Deleted story - not available for reading/searching') ;

# --------------------------------------------------------

#
# Table structure for table `cs_topics`
#

CREATE TABLE cs_topics (
  id smallint NOT NULL auto_increment,
  topic varchar(255) NOT NULL,
  PRIMARY KEY  (id),
  UNIQUE KEY topic (topic)
) COMMENT='Optional post categories';

# --------------------------------------------------------

#
# Table structure for table `cs_story`
#

CREATE TABLE cs_story (
  id integer unsigned NOT NULL auto_increment,
  author_id smallint NOT NULL,
  title varchar(255),
  abstract text,
  story text,
  extended_story text,
  created_on datetime NOT NULL,
  updated_on datetime NOT NULL,
  scheduled_for_date date,
  status tinyint NOT NULL default '1',
  allow_comments enum('1','0') NOT NULL default '1',
  allow_pings enum('1','0') NOT NULL default '1',
  PRIMARY KEY  (id),
  FULLTEXT KEY abstract (abstract)
) TYPE=MyISAM;

# --------------------------------------------------------

#
# Table structure for table `cs_story_topics`
#

CREATE TABLE cs_story_topics (
  story_id integer NOT NULL default '0',
  topic_id smallint NOT NULL default '0',
  PRIMARY KEY  (story_id,topic_id)
) TYPE=MyISAM;

# --------------------------------------------------------

#
# Table structure for table `cs_story_reference`
#

CREATE TABLE cs_story_reference (
  rowid integer unsigned NOT NULL auto_increment,
  story_id integer unsigned NOT NULL default '0',
  reference_url varchar(255) NOT NULL,
  reference_ping varchar(255),
  reference_title varchar(255),
  created_on datetime NOT NULL,
  pinged_on datetime,
  PRIMARY KEY  (rowid)
) COMMENT='Post cross-references (enhanced trackback)';

# --------------------------------------------------------

#
# Table structure for table `cs_comment`
#

CREATE TABLE cs_comment (
  id integer unsigned NOT NULL auto_increment,
  story_id integer unsigned NOT NULL,
  created_on datetime NOT NULL,
  guest varchar(255) NOT NULL,
  email varchar(255),
  url varchar(255),
  comment text NOT NULL,
  ip varchar(32) default 'unknown',
  country varchar(255) default 'unknown',
  PRIMARY KEY  (id)
) TYPE=MyISAM;

# --------------------------------------------------------

#
# Table structure for table `cs_trackback`
#

CREATE TABLE cs_trackback (
  id integer unsigned NOT NULL auto_increment,
  story_id integer unsigned NOT NULL default '0',
  created_on datetime NOT NULL,
  remote_blog varchar(255),
  url varchar(255) NOT NULL,
  title varchar(255),
  comment varchar(255) NOT NULL,
  PRIMARY KEY  (id)
) TYPE=MyISAM;

# --------------------------------------------------------

#
# Table structure for table `cs_linkscat`
#

CREATE TABLE cs_linkscat (
  id tinyint unsigned NOT NULL auto_increment,
  category varchar(128) NOT NULL,
  rank tinyint NOT NULL default '1',
  PRIMARY KEY  (id),
  UNIQUE KEY category (category)
) COMMENT='Link categories';

# default Link category
INSERT INTO cs_linkscat (id, category, rank) VALUES (1, "General", 1) ;
# --------------------------------------------------------

#
# Table structure for table `cs_neighborhood`
#

CREATE TABLE cs_neighborhood (
  id smallint NOT NULL auto_increment,
  cat_id tinyint unsigned,
  uri varchar(255) NOT NULL,
  short_name varchar(255) NOT NULL,
  long_name varchar(255),
  rss_file varchar(255),
  created_on datetime NOT NULL,
  active tinyint NOT NULL default '1',
  rank smallint,
  PRIMARY KEY  (id)
) TYPE=MyISAM;

# --------------------------------------------------------

#
# Table structure for table `cs_banlist`
#

CREATE TABLE cs_banlist (
  rowid integer unsigned NOT NULL auto_increment,
  ip varchar(32) NOT NULL,
  country varchar(255) NOT NULL,
  created_on datetime NOT NULL,
  PRIMARY KEY  (rowid),
  UNIQUE KEY ip (ip)
) COMMENT='Banned IPs here';

# --------------------------------------------------------

#
# Table structure for table `cs_logbook`
#

CREATE TABLE cs_logbook (
  rowid integer unsigned NOT NULL auto_increment,
  action text NOT NULL,
  created_on datetime NOT NULL,
  PRIMARY KEY  (rowid)
) COMMENT='System logs';

# --------------------------------------------------------

#
# Table structure for table `cs_rpc_service`
#

CREATE TABLE cs_rpc_service (
  service_name varchar(255) NOT NULL,
  remote_server varchar(255) NOT NULL,
  remote_path varchar(255) NOT NULL,
  remote_method varchar(255) NOT NULL default 'weblogUpdates.ping',
  PRIMARY KEY  (service_name)
) COMMENT='List of RPC services accepting update pings';

# known PING services
INSERT INTO cs_rpc_service (service_name, remote_server, remote_path, remote_method) VALUES ('Weblogs.com', 'rpc.weblogs.com', '/RPC2', 'weblogUpdates.ping') ;
INSERT INTO cs_rpc_service (service_name, remote_server, remote_path, remote_method) VALUES ('Blo.gs', 'ping.blo.gs', '/', 'weblogUpdates.extendedPing') ;
INSERT INTO cs_rpc_service (service_name, remote_server, remote_path, remote_method) VALUES ('Blogrolling', 'rpc.blogrolling.com', '/pinger/', 'weblogUpdates.ping') ;
INSERT INTO cs_rpc_service (service_name, remote_server, remote_path, remote_method) VALUES ('Rootblog', 'ping.rootblog.com', '/rpc.php', 'weblogUpdates.ping') ;
INSERT INTO cs_rpc_service (service_name, remote_server, remote_path, remote_method) VALUES ('Wordsmith', 'tomaskohl.com', '/wordsmith/xserver.php', 'weblogUpdates.ping') ;

# --------------------------------------------------------

#
# Table structure for table `cs_scrapbook`
#

CREATE TABLE cs_scrapbook (
  id smallint unsigned NOT NULL auto_increment,
  author_id smallint NOT NULL default '0',
  topic_id smallint,
  note text NOT NULL,
  created_on datetime NOT NULL,
  published_on datetime,
  PRIMARY KEY  (id)
) TYPE=MyISAM;

# --------------------------------------------------------

#
# Table structure for table `cs_session`
#

CREATE TABLE cs_session (
  id char(32) NOT NULL,
  user_id smallint NOT NULL default '0',
  created_on datetime NOT NULL,
  valid_until datetime NOT NULL,
  PRIMARY KEY  (id)
) TYPE=MyISAM;

# --------------------------------------------------------

#
# Table structure for table `cs_tracker`
#

CREATE TABLE cs_tracker (
  rowid integer unsigned NOT NULL auto_increment,
  page varchar(255) NOT NULL,
  ip varchar(32),
  country varchar(255),
  hostname varchar(255),
  referer varchar(255),
  story_id integer unsigned NOT NULL,
  created_on datetime NOT NULL,
  PRIMARY KEY  (rowid)
) COMMENT='Very simple tracker';

# --------------------------------------------------------

#
# Table structure for table `cs_stats`
#

CREATE TABLE cs_stats (
  rowid integer unsigned NOT NULL auto_increment,
  page varchar(255) NOT NULL,
  ip varchar(32),
  country varchar(255),
  hostname varchar(255),
  referer varchar(255),
  story_id integer unsigned NOT NULL,
  created_on datetime NOT NULL,
  updated_on date,
  PRIMARY KEY  (rowid)
) COMMENT='Aggregated tracker data for statistical purposes';

# --------------------------------------------------------